package com.caimei365.tools.model.dto.message;

import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;

@Data
@Accessors(fluent = true)
@AllArgsConstructor
@NoArgsConstructor
public class WechatTemplateMessage {
    /**
     *  {
     *            "touser":"OPENID",
     *            "template_id":"ngqIpbwh8bUfcSsECmogfXcV14J0tQlEpBO27izEYtY",
     *            "url":"http://weixin.qq.com/download",
     *            "miniprogram":{
     *              "appid":"xiaochengxuappid12345",
     *              "pagepath":"index?foo=bar"
     *            },
     *            "client_msg_id":"MSG_000001",
     *            "data":{
     *                    "first": {
     *                        "value":"恭喜你购买成功！",
     *                        "color":"#173177"
     *                    },
     *                    "keyword1":{
     *                        "value":"巧克力",
     *                        "color":"#173177"
     *                    },
     *                    "keyword2": {
     *                        "value":"39.8元",
     *                        "color":"#173177"
     *                    },
     *                    "keyword3": {
     *                        "value":"2014年9月22日",
     *                        "color":"#173177"
     *                    },
     *                    "remark":{
     *                        "value":"欢迎再次购买！",
     *                        "color":"#173177"
     *                    }
     *            }
     *        }
     */

    /**
     * 推送内容抬头
     */
    @NotNull
    private JSONObject first;
    /**
     * 推送内容
     */
    @NotNull
    private List<JSONObject> keyWords;
    /**
     * 推送内容页尾
     */
    @NotNull
    private JSONObject remark;
    /**
     * 所需跳转到的小程序appid（该小程序 appid 必须与发模板消息的公众号是绑定关联关系，暂不支持小游戏）
     */
    private String appid;

    /**
     * 接收者openid
     */
    @NotNull
    private String openId;
    /**
     * 模板ID
     */
    @NotNull
    private String templateId;
    /**
     * 模板跳转链接
     */
    private String url;
    /**
     * 所需跳转到小程序的具体页面路径，支持带参数,（示例index?foo=bar），要求该小程序已发布，暂不支持小游戏
     */
    private String pagePath;


    /**
     * 跳小程序所需数据，不需跳小程序可不用传该数据
     */
    private JSONObject miniProgram;
    /**
     * 模板数据
     */
    private JSONObject data;
    private JSONObject json;

    public JSONObject getFirst() {
        return first;
    }

    public void setFirst(JSONObject first) {
        this.first = first;
    }

    public List<JSONObject> getKeyWords() {
        return keyWords;
    }


    public JSONObject getRemark() {
        return remark;
    }

    public void setRemark(JSONObject remark) {
        this.remark = remark;
    }

    public String getAppid() {
        return appid;
    }

    public void setAppid(String appid) {
        this.appid = appid;
    }

    public String getOpenId() {
        return openId;
    }

    public void setOpenId(String openId) {
        this.openId = openId;
    }

    public String getTemplateId() {
        return templateId;
    }

    public void setTemplateId(String templateId) {
        this.templateId = templateId;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public String getPagePath() {
        return pagePath;
    }

    public void setPagePath(String pagePath) {
        this.pagePath = pagePath;
    }

    public WechatTemplateMessage first(String title) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("value", title);
        this.setFirst(jsonObject);
        return this;
    }

    public WechatTemplateMessage keyWords(List<String> words) {
        List<JSONObject> collect = new ArrayList<>();
        words.forEach(w -> {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("value", w);
            collect.add(jsonObject);
        });
        this.setKeyWords(collect);
        return this;
    }

    public void setKeyWords(List<JSONObject> keyWords) {
        this.keyWords = keyWords;
    }

    public WechatTemplateMessage remark(String r) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("value", r);
        this.setRemark(jsonObject);
        return this;
    }

    public JSONObject getMiniProgram() {
        return miniProgram;
    }

    private void setMiniProgram(JSONObject miniProgram) {
        this.miniProgram = miniProgram;
    }

    private JSONObject getData() {
        return data;
    }

    private void setData(JSONObject data) {
        this.data = data;
    }

    public String getJson() {
        JSONObject json = new JSONObject();
        json.put("touser", openId);
        json.put("template_id", templateId);
        json.put("url", url);

        JSONObject miniProgram = new JSONObject();
        miniProgram.put("appid", appid);
        miniProgram.put("pagepath", pagePath);
        json.put("miniprogram", miniProgram);

        JSONObject data = new JSONObject();
        data.put("first", first);
        if (!keyWords.isEmpty()) {
            for (int i = 0; i < keyWords.size(); i++) {
                String key = "keyword" + (i + 1);
                data.put(key, keyWords.get(i));
            }
        }
        data.put("remark", remark);
        json.put("data", data);
        return json.toJSONString();
    }
}
