package com.caimei365.tools.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.tools.mapper.CmBehaviorRecordMapper;
import com.caimei365.tools.model.po.CmBehaviorRecordPo;
import com.caimei365.tools.model.po.IpSavePo;
import com.caimei365.tools.service.CmBehaviorRecordService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/11/2
 */
@Slf4j
@Service
public class CmBehaviorRecordServiceImpl implements CmBehaviorRecordService {

    @Autowired private CmBehaviorRecordMapper recordMapper;

    @Override
    public void insertRecord(IpSavePo ipSavePo) {
        if ("/user/record/Statistics".equals(ipSavePo.getRequestUrl().trim())) {
            log.info("用户行为记录=================》" + ipSavePo.getIp());
            CmBehaviorRecordPo cmBehaviorRecordPo = new CmBehaviorRecordPo();
            String region = "";
            cmBehaviorRecordPo.setIP(ipSavePo.getIp());
            if (StringUtils.isNotBlank(ipSavePo.getUserId())) {
                cmBehaviorRecordPo.setUserId(ipSavePo.getUserId());
            } else {
                cmBehaviorRecordPo.setUserId("0");
            }
            if (ipSavePo.getParams().contains("userId") || ipSavePo.getParams().contains("productId")) {
                String[] split = ipSavePo.getParams().split("&");
                for (String s : split) {
                    String str = s.trim();
                    // 页面路径
                    if (str.startsWith("pagePath") && str.contains("=")) {
                        String[] split1 = str.split(str.substring(str.indexOf("="),str.indexOf("=")+1),2);
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                String decode = null;
                                try {
                                    decode = java.net.URLDecoder.decode(trim,"UTF-8");
                                    if (decode.contains("%")) {
                                        decode = java.net.URLDecoder.decode(decode, "UTF-8");
                                    }
                                } catch (UnsupportedEncodingException e) {
                                    e.printStackTrace();
                                }
                                cmBehaviorRecordPo.setPagePath(decode);
                            }
                            if (StringUtils.isEmpty(cmBehaviorRecordPo.getPagePath())) {
                                log.info("页面路径pagePath 值为====》" + trim);
                                return ;
                            }
                        }
                    }
                    // 页面类型
                    if (str.startsWith("pageType") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        // 赋默认值
                        cmBehaviorRecordPo.setPageType(0);
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                int parseInt = Integer.parseInt(trim);

                                cmBehaviorRecordPo.setPageType(parseInt);
                            }
                        }
                    }
                    // 页面标签
                    if (str.startsWith("pageLabel") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        cmBehaviorRecordPo.setPageLabel("");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {

                                // 接口参数解码
                                String decode = null;
                                try {
                                    decode = java.net.URLDecoder.decode(trim, "UTF-8");
                                    if (decode.contains("%")) {
                                        decode = java.net.URLDecoder.decode(decode, "UTF-8");
                                    }
                                } catch (UnsupportedEncodingException e) {
                                    e.printStackTrace();
                                }
                                cmBehaviorRecordPo.setPageLabel(decode);
                            }
                        }
                    }
                    // 用户行为记录 or roos页面统计
                    if (str.startsWith("behaviorType") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        cmBehaviorRecordPo.setBehaviorType("1");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                if (!trim.equals("null")) {
                                    cmBehaviorRecordPo.setBehaviorType(trim);
                                } else {
                                    cmBehaviorRecordPo.setBehaviorType("0");
                                }

                            }
                        }
                    }
                    // 商品ID
                    if (str.startsWith("productId") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                cmBehaviorRecordPo.setProductId(trim);
                            }
                        }
                    }
                    // 访问客户端 0：网站 1：小程序
                    if (str.startsWith("accessClient") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        cmBehaviorRecordPo.setAccessClient("0");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                cmBehaviorRecordPo.setAccessClient(trim);
                            }
                        }
                    }
                    // 浏览器链接
                    if (str.startsWith("referer") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
//                                log.info("referer============》"+trim);
                                // 浏览器链接
                                cmBehaviorRecordPo.setReferer(trim);
                                // 访问来源
                                String source = source(trim);
                                cmBehaviorRecordPo.setAccessSource(source);
                            }
                        }
                    }
                    // 设置删除标记 判定爬虫
                    cmBehaviorRecordPo.setDelFlag("0");
                    // 浏览器userAgent
                    if (str.startsWith("userAgent") && str.contains("=")) {
                        String[] split1 = str.split("=");
                        if (split1.length > 1) {
                            String value = split1[1];
                            String trim = value.trim();
                            if (StringUtils.isNotBlank(trim)) {
                                log.info("userAgent============》"+trim);
                                // 浏览器userAgent
                                cmBehaviorRecordPo.setUserAgent(trim);
                                if (boolReptiles(trim)) {
                                    cmBehaviorRecordPo.setDelFlag("1");
                                }
                            }
                        }
                    }
                }
            }

            Date date = new Date();
            cmBehaviorRecordPo.setAccessTime(date);
            cmBehaviorRecordPo.setAccessDuration("5000");
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
            String format = simpleDateFormat.format(date);
            cmBehaviorRecordPo.setAccessDate(format);
            cmBehaviorRecordPo.setIsReckon("0");
            // 确定ip所属地

            try {
                region = recordIp(ipSavePo.getIp());
            } catch (IOException e) {
                e.printStackTrace();
            }
            cmBehaviorRecordPo.setRegion(region);
            recordMapper.insertRecord(cmBehaviorRecordPo);
        }
    }

    @Override
    public void updateBehaviorTime() {
        log.info("计算实际访问时长==================》");
        long t1 = 0;
        long t2 = 0;
        Long duration = 0L;
        CmBehaviorRecordPo cmBehaviorRecordPo = new CmBehaviorRecordPo();
        CmBehaviorRecordPo cmBehaviorRecord = new CmBehaviorRecordPo();
        Date date = new Date();
        // 获取当前时间日期
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String format = simpleDateFormat.format(new Date(date.getTime() - 600000));
        cmBehaviorRecordPo.setAccessDate(format);
        // 当天访问系统ip和userID
        List<CmBehaviorRecordPo> recordPos = recordMapper.toDateRecode(cmBehaviorRecordPo);
        for (CmBehaviorRecordPo behaviorRecordPo: recordPos) {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            // 当前时间
            String timeEnd = dateFormat.format(date);
            //  一小时前时间
            Calendar c = Calendar.getInstance();
            c.set(Calendar.HOUR_OF_DAY,(c.get(Calendar.HOUR_OF_DAY) - 1));
            String timeStart = dateFormat.format(c.getTime());

            behaviorRecordPo.setStartTime(timeStart);
            behaviorRecordPo.setEndTime(timeEnd);
            // 获取一个小时之内的数据
            List<CmBehaviorRecordPo> recordPoList = recordMapper.toDateIPTimeRecode(behaviorRecordPo);
            if (recordPoList.size() <= 0) {
                continue;
            }
            for (int i = 0; i < recordPoList.size() ; i++) {
                if (i != recordPoList.size() - 1) {
                    t1 = recordPoList.get(i).getAccessTime().getTime();
                    t2 = recordPoList.get(i+1).getAccessTime().getTime();
                    duration = t2 - t1;
                    // 修改实际访问时长
                    cmBehaviorRecord.setRecordID(recordPoList.get(i).getRecordID());
                    cmBehaviorRecord.setAccessDuration(duration.toString());
                    cmBehaviorRecord.setIsReckon("1");
                    recordMapper.updateDuration(cmBehaviorRecord);
                }
            }
        }
    }

    @Override
    public void setIpAddress(Integer startId, Integer endId) {
        List<CmBehaviorRecordPo> list = recordMapper.findList(startId,endId);
        String region = "";
        for (CmBehaviorRecordPo recordPo: list) {
            try {
                // 获取ip所在地
                region = recordIp(recordPo.getIP());
                recordPo.setRegion(region);
                recordPo.setIsReckon("1");
                recordMapper.updateOldData(recordPo);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void updateBehaviorUser() {
        log.info("====合并IP====");
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH,-1);
        Date time = calendar.getTime();
        String format = dateFormat.format(time);
        // 前一天数据身份中是否只有 机构-游客、供应商-游客
        List<CmBehaviorRecordPo> countNumList = recordMapper.countNum(format);
        if (countNumList.size() != 0 && countNumList != null) {
            for (CmBehaviorRecordPo countNum: countNumList) {
                if (countNum.getCount() == 2) {
                    // 查询机构/供应商ip、userID，用于调整游客数据
                    CmBehaviorRecordPo recordPo = recordMapper.userIdAcc(countNum.getIP(), format);
                    // 查询游客数据
                    List<CmBehaviorRecordPo> behaviorList = recordMapper.selBehaviorList(countNum.getIP(), format);
                    // 将游客数据中的ip和userId修改为机构/供应商数据
                    for (CmBehaviorRecordPo cmBehaviorRecordPo : behaviorList) {
                        cmBehaviorRecordPo.setIP(recordPo.getIP());
                        cmBehaviorRecordPo.setUserId(recordPo.getUserId());
                        recordMapper.updateBehavior(cmBehaviorRecordPo);
                    }
                }
            }
        }
    }

    /**
     * 昨日用户行为列表数据生成
     */
    @Override
    public void insertBehaviorUser() {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH,-1);
        Date time = calendar.getTime();
        String format = dateFormat.format(time);

        // 获取昨天数据
        recordMapper.selYesterdayList(format);
        log.info("====昨日用户行为列表数据已生成====");
    }


    // 获取IP对应地址 ---- 太平洋
    public static String recordIp(String ip) throws IOException {
        URL url = null;
        HttpURLConnection connection = null;
        String encoding = "gbk";
        String text = "";
        String line = "";
        String region = "";
        String urlStr = "http://whois.pconline.com.cn/ipJson.jsp?ip="+ip+"&json=true";
        try {
            url = new URL(urlStr);
            connection = (HttpURLConnection) url.openConnection();	//新建链接实例
            connection.setConnectTimeout(20000);	//设置链接超时时间，单位毫秒
            connection.setReadTimeout(20000);		//设置读取数据超时时间，单位毫秒
            connection.setDoOutput(true);		//是否打开输出流true|false
            connection.setDoInput(true);		//是否打开输入流true|false
            connection.setRequestMethod("GET");		// 提交方式get|post
            connection.setUseCaches(false);		// 是否加入缓存true|false
            connection.connect();	//打开链接端口

            BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream(), encoding));	// 往对端写完数据对端服务器返回数据。以BufferedReader流来读取

            while ((line = reader.readLine()) != null) {
                text += line+"\n";
            }
            reader.close();

            JSONObject jsonObject = JSONObject.parseObject(text);
            region = jsonObject.get("addr").toString();
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } finally {
            if (connection != null) {
                connection.disconnect();	//关闭连接
            }
        }
        return region;
    }

    private String source(String link) {
        if (link.contains("www.baidu.com")) {
            return "1";
        }
        if (link.contains("www.so.com")) {
            return "2";
        }
        if (link.contains("www.google.cn")) {
            return "3";
        }
        if (link.contains("m.sm.cn")) {
            return "4";
        }
        if (link.contains("toutiao.com")) {
            return "5";
        }
        if (link.contains("www.sogou.com")) {
            return "6";
        }
        if (link.contains("servicewechat.com")) {
            return "7";
        }
        if (link.contains("www.caimei365.com")) {
            return "0";
        }
        if (link.contains("zzjtest.gz.aeert.com")) {
            return "0";
        }
        return null;
    }
    private boolean boolReptiles(String userAgent) {
        if (userAgent.contains("serpstatbot") || userAgent.contains("spider") || userAgent.contains("Googlebot") || userAgent.contains("bingbot")
            || userAgent.contains("SeznamBot") || userAgent.contains("YisouSpider") || userAgent.contains("petalbot") || userAgent.contains("YandexBot")
            || userAgent.contains("AhrefsBot")) {
            return true;
        }
        return false;
    }
}
